from datetime import datetime, timezone
from enum import Enum
from typing import List, Optional
from uuid import uuid4

from pydantic import BaseModel, Field


def generate_uuid() -> str:
    return str(uuid4())


def generate_datetime_str() -> str:
    return (datetime.utcnow()).replace(tzinfo=timezone.utc).isoformat()


class TestStatus(Enum):
    UNKNOWN = "UNKNOWN"
    IN_PROGRESS = "IN_PROGRESS"
    PASSED = "PASSED"
    FAILED = "FAILED"
    SKIPPED = "SKIPPED"
    ABORTED = "ABORTED"
    QUEUED = "QUEUED"


def to_camel_case(key: str) -> str:
    parts = key.split("_")
    for i, part in enumerate(parts[1:]):
        parts[i + 1] = part.capitalize()

    return "".join(parts)


class CamelModel(BaseModel):
    class Config:
        alias_generator = to_camel_case
        allow_population_by_field_name = True


class TestRunConfigModel(CamelModel):
    environment: Optional[str] = None
    build: Optional[str] = None


class StartTestRunModel(CamelModel):
    name: str
    framework: str
    started_at: str = Field(default_factory=generate_datetime_str)
    uuid: str = Field(default_factory=generate_uuid)
    config: Optional[TestRunConfigModel] = None


class LabelModel(CamelModel):
    key: str
    value: str


class StartTestModel(CamelModel):
    name: str
    class_name: str
    method_name: str
    uuid: str = Field(default_factory=generate_uuid)
    started_at: str = Field(default_factory=generate_datetime_str)
    maintainer: Optional[str] = None
    test_case: Optional[str] = None
    labels: Optional[List[LabelModel]]


class FinishTestModel(CamelModel):
    result: str
    ended_at: str = Field(default_factory=generate_datetime_str)
    reason: Optional[str] = None


class LogRecordModel(CamelModel):
    test_id: str
    level: str
    timestamp: str
    message: str


class StartTestSessionModel(CamelModel):
    session_id: str
    started_at: str = Field(default_factory=generate_datetime_str)
    desired_capabilities: dict
    capabilities: dict
    test_ids: List[str] = []


class FinishTestSessionModel(CamelModel):
    ended_at: str = Field(default_factory=generate_datetime_str)
    test_ids: List[str] = []


class ArtifactReferenceModel(CamelModel):
    name: str
    value: str


if __name__ == "__main__":
    print()



from pathlib import Path
from typing import Union

from pytest_zebrunner.api.client import ZebrunnerAPI
from pytest_zebrunner.api.models import ArtifactReferenceModel, LabelModel
from pytest_zebrunner.context import zebrunner_context

api = ZebrunnerAPI()


def attach_test_screenshot(path: Union[str, Path]) -> None:
    if zebrunner_context.test_is_active:
        api.send_screenshot(zebrunner_context.test_run_id, zebrunner_context.test_id, path)


def attach_test_artifact(path: Union[str, Path]) -> None:
    if zebrunner_context.test_is_active:
        api.send_artifact(path, zebrunner_context.test_run_id, zebrunner_context.test_id)


def attach_test_run_artifact(path: Union[str, Path]) -> None:
    if zebrunner_context.test_run_is_active:
        api.send_artifact(path, zebrunner_context.test_run_id)


def attach_test_artifact_reference(name: str, ref: str) -> None:
    if zebrunner_context.test_is_active:
        api.send_artifact_references(
            [ArtifactReferenceModel(name=name, value=ref)], zebrunner_context.test_run_id, zebrunner_context.test_id
        )


def attach_test_run_artifact_reference(name: str, ref: str) -> None:
    if zebrunner_context.test_run_is_active:
        api.send_artifact_references([ArtifactReferenceModel(name=name, value=ref)], zebrunner_context.test_run_id)


def attach_test_label(name: str, value: str) -> None:
    if zebrunner_context.test_is_active:
        api.send_labels([LabelModel(key=name, value=value)], zebrunner_context.test_run_id, zebrunner_context.test_id)


def attach_test_run_label(name: str, value: str) -> None:
    if zebrunner_context.test_run_is_active:
        api.send_labels([LabelModel(key=name, value=value)], zebrunner_context.test_run_id)



from typing import Optional

from pydantic import BaseSettings


class ZebrunnerSettings(BaseSettings):
    service_url: str
    access_token: str
    zebrunner_project: str
    reporting_enabled: bool = True
    build: Optional[str] = None
    test_run_name: Optional[str]
    env: Optional[str] = None

    class Config:
        env_file = ".env"



import time
from logging import LogRecord, StreamHandler
from typing import List

from pytest_zebrunner.api.client import ZebrunnerAPI
from pytest_zebrunner.api.models import LogRecordModel
from pytest_zebrunner.context import zebrunner_context


class ZebrunnerHandler(StreamHandler):
    BATСH_SIZE = 20
    logs: List[LogRecordModel] = []

    def __init__(self) -> None:
        super().__init__()
        self.api = ZebrunnerAPI()

    def emit(self, record: LogRecord) -> None:
        if len(self.logs) >= self.BATСH_SIZE:
            self.push_logs()

        if zebrunner_context.test_is_active:
            self.logs.append(
                LogRecordModel(
                    test_id=str(zebrunner_context.test_id),
                    timestamp=str(round(time.time() * 1000)),
                    level=record.levelname,
                    message=record.message,
                )
            )

    def push_logs(self) -> None:
        logs = self.logs
        self.logs = []

        if zebrunner_context.test_run_id:
            self.api.send_logs(zebrunner_context.test_run_id, logs)



import logging
from typing import Union

import pytest
from _pytest.config import Config, ExitCode
from _pytest.main import Session
from _pytest.nodes import Item
from _pytest.reports import TestReport
from _pytest.runner import CallInfo

from pytest_zebrunner.api.client import ZebrunnerAPI
from pytest_zebrunner.api.models import (
    FinishTestModel,
    StartTestModel,
    StartTestRunModel,
    TestRunConfigModel,
    TestStatus,
)
from pytest_zebrunner.context import Test, TestRun, zebrunner_context
from pytest_zebrunner.selenium_integration import SeleniumSession, inject_driver
from pytest_zebrunner.zebrunner_logging import ZebrunnerHandler

logger = logging.getLogger(__name__)


class PytestZebrunnerHooks:
    def __init__(self) -> None:
        self.api = ZebrunnerAPI(zebrunner_context.settings.service_url, zebrunner_context.settings.access_token)
        self.session_manager = SeleniumSession(self.api)

    @pytest.hookimpl
    def pytest_sessionstart(self, session: Session) -> None:
        self.api.auth()
        inject_driver(self.session_manager)
        settings = zebrunner_context.settings
        test_run = TestRun(settings.test_run_name, settings.env, settings.build)

        zebrunner_context.test_run = test_run
        test_run.zebrunner_id = self.api.start_test_run(
            settings.zebrunner_project,
            StartTestRunModel(
                name=test_run.name,
                framework="pytest",
                config=TestRunConfigModel(environment=test_run.environment, build=test_run.build),
            ),
        )
        logging.root.addHandler(ZebrunnerHandler())

    @pytest.hookimpl
    def pytest_sessionfinish(self, session: Session, exitstatus: Union[int, ExitCode]) -> None:
        if zebrunner_context.test_run_is_active:
            self.api.finish_test_run(zebrunner_context.test_run_id)
            self.session_manager.finish_all_sessions()

            handlers = list(filter(lambda x: isinstance(x, ZebrunnerHandler), logging.root.handlers))
            if len(handlers) > 0:
                zebrunner_handler: ZebrunnerHandler = handlers[0]  # type: ignore
                zebrunner_handler.push_logs()

        self.api.close()

    @pytest.hookimpl
    def pytest_runtest_makereport(self, item: Item, call: CallInfo) -> TestReport:
        report = TestReport.from_item_and_call(item, call)
        report.item = item  # type: ignore
        return report

    @pytest.hookimpl
    def pytest_report_teststatus(self, report: TestReport, config: Config) -> None:
        if report.when == "setup":
            self.setup_test(report)
        elif report.when == "call":
            self.call_test(report)

    def setup_test(self, report: TestReport) -> None:
        test_item: Item = report.item
        test = Test(
            name=test_item.name,
            file=test_item.nodeid.split("::")[1],
            maintainers=[mark.args[0] for mark in test_item.iter_markers("maintainer")],
            labels=[(str(mark.args[0]), str(mark.args[1])) for mark in test_item.iter_markers("label")],
        )
        zebrunner_context.test = test

        if zebrunner_context.test_run_is_active:
            test.zebrunner_id = self.api.start_test(
                zebrunner_context.test_run_id,
                StartTestModel(
                    name=test.name,
                    class_name=test.file,
                    method_name=test.name,
                    maintainer=",".join(test.maintainers),
                    labels=[{"key": label[0], "value": label[1]} for label in test.labels],
                ),
            )
            self.session_manager.add_test(zebrunner_context.test_id)

        if report.skipped and zebrunner_context.test_is_active:
            skip_markers = list(filter(lambda x: x.name == "skip", report.item.own_markers))
            skip_reason = skip_markers[0].kwargs.get("reason") if skip_markers else None
            self.api.finish_test(
                zebrunner_context.test_run_id,
                zebrunner_context.test_id,
                FinishTestModel(reason=skip_reason, result=TestStatus.SKIPPED.value),
            )
            zebrunner_context.test = None

    def call_test(self, report: TestReport) -> None:
        if zebrunner_context.test_is_active:
            if report.skipped:
                self.api.finish_test(
                    zebrunner_context.test_run_id,
                    zebrunner_context.test_id,
                    FinishTestModel(result=TestStatus.SKIPPED.value, reason=report.wasxfail or None),
                )
                zebrunner_context.test = None
                return

            if report.passed:
                status = TestStatus.PASSED
            else:
                status = TestStatus.FAILED

            body = FinishTestModel(result=status.value)
            self.api.finish_test(zebrunner_context.test_run_id, zebrunner_context.test_id, body)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pytest_zebrunner', 'pytest_zebrunner.api']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.16.1,<0.17.0',
 'pydantic>=1.7.2,<2.0.0',
 'pytest>=6.1.1,<7.0.0',
 'python-dotenv>=0.15.0,<0.16.0']

entry_points = \
{'pytest11': ['pytest-zebrunner = pytest_zebrunner.plugin']}

setup_kwargs = {
    'name': 'pytest-zebrunner',
    'version': '0.1.0',
    'description': 'Pytest connector for Zebrunner reporting',
    'long_description': '# Zebrunner PyTest agent\n\n\nThe official Zebrunner Pytest agent provides reporting functionality. It can automatically track selenium sessions\nand send info about session details to Zebrunner backend. It can be ease integrated in project just by installing library\nand adding configuration file.\n\n\n## Installation\n---------------\n\n    pip install pytest-zebrunner\n\n## Configuration\n----------------\nLibrary don\'t ready for usage just after installation. It won\'t send data to\nzebrunner service without additional configuration. To configure app you need to\nspecify environment variables. It can be done by specifying variables in `.env`\nfile.\n\n<!-- groups:start -->\n### Environment variables\n```dosini\nSERVICE_URL=<zebrunner url>\nACCESS_TOKEN=<access_token>\nZEBRUNNER_PROJECT=ProjectName\nZEBRUNNER_ENABLED=true\nBUILD=1.25.16\nENV=stage\n```\n\nYou can configure agent only with environment variables. Another formats would\nbe added in future. Planed formats are `yaml`, `ini` and program arguments.\n\n- `SERVICE_URL` - [required] Base URL of Zebrunner service.\n\n- `ACCESS_TOKEN` - [required] Authorization token. You can find it in Account & profile section.\n\n- `ZEBRUNNER_PROJECT` - [required] Name of project.\n\n- `REPORTING_ENABLED` - You can disable agent if it makes side effects in you project or doesn\'t work.\n\n- `BUILD` - Version of product on which tests are running.\n\n- `ENV` - Testing environment name.\n<!-- groups:end -->\n\nIf required configurations not provided there is a warning in logs with problem description and names of options,\nwhich need to be specified. Parameter names are case insensitive and can be written in upper and lower registers.\n\n## Additional functionality\n---------------------------\n**IMPORTANT**: All attachments to tests can be done only while some test is running. All attachments to test-run can be done only while pytest test-session is active.\n---------------------------\n\n\n### Attach screenshot\n```python\nfrom pytest_zebrunner.attachments import attach_test_screenshot\n\n\ndef test_something():\n    ...\n    attach_test_screenshot("path_to_screenshot.png")\n    ...\n```\n\n### Attach artifact to test\n```python\nfrom pytest_zebrunner.attachments import attach_test_artifact\n\n\ndef test_something():\n    ...\n    attach_test_artifact("path_to_artifact")\n    ...\n```\n\n### Attach artifact-reference to test\n```python\nfrom pytest_zebrunner.attachments import attach_test_artifact_reference\n\n\ndef test_something():\n    ...\n    attach_test_artifact_reference("name", "reference")\n    ...\n```\n\n### Attach artifact to test-run\n```python\nfrom pytest_zebrunner.attachments import attach_test_run_artifact\n\n\nattach_test_run_artifact("path_to_artifact")\n```\n\n### Attach artifact-reference to test-run\n```python\nfrom pytest_zebrunner.attachments import attach_test_run_artifact_reference\n\n\nattach_test_run_artifact_reference("name", "reference")\n```\n\n### Attach labels to test\n```python\n@pytest.mark.label("name", "value")\ndef test_something():\n    ...\n```\nor\n```python\nfrom pytest_zebrunner.attachments import attach_test_label\n\n\ndef test_something():\n    ...\n    attach_test_label("name", "value")\n    ...\n```\n**Note:** These two methods can be combined.\n\n### Attach label to test run\n```python\nfrom pytest_zebrunner.attachments import attach_test_run_label\n\nattach_test_run_label("name", "value")\n```\n\n### Add maintainer to test\n```python\n@pytest.mark.maintainer("username_of_maintainer")\ndef test_something():\n    ...\n```\n',
    'author': 'Anatoliy Platonov',
    'author_email': 'p4m.dev@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://zebrunner.com/documentation/agents/pytest',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import logging
from datetime import datetime, timedelta, timezone
from pathlib import Path
from pprint import pformat
from typing import List, Optional, Union

import httpx
from httpx import Client, Request, Response

from pytest_zebrunner.api.models import (
    ArtifactReferenceModel,
    FinishTestModel,
    FinishTestSessionModel,
    LabelModel,
    LogRecordModel,
    StartTestModel,
    StartTestRunModel,
    StartTestSessionModel,
)
from pytest_zebrunner.utils import Singleton

logger = logging.getLogger(__name__)


def log_response(response: Response, log_level: int = logging.DEBUG) -> None:
    request = response.request
    request.read()

    logger.log(
        log_level,
        f"Request {request.method} {request.url}\n"
        f"Headers: \n{pformat(dict(request.headers))}\n\n"
        f"Content: \n{request.content}\n\n"
        f"Response Code: {response.status_code}\n"
        f" Content: \n{pformat(response.json())}",
    )


class ZebrunnerAPI(metaclass=Singleton):
    authenticated = False

    def __init__(self, service_url: str = None, access_token: str = None):
        if service_url and access_token:
            self.service_url = service_url.rstrip("/")
            self.access_token = access_token
            self._client = Client()
            self._auth_token = None
            self.authenticated = False

    def _sign_request(self, request: Request) -> Request:
        request.headers["Authorization"] = f"Bearer {self._auth_token}"
        return request

    def auth(self) -> None:
        if not self.access_token or not self.service_url:
            return

        url = self.service_url + "/api/iam/v1/auth/refresh"

        try:
            response = self._client.post(url, json={"refreshToken": self.access_token})
        except httpx.RequestError as e:
            logger.warning("Error while sending request to zebrunner.", exc_info=e)
            return

        if response.status_code != 200:
            log_response(response, logging.ERROR)
            return

        self._auth_token = response.json()["authToken"]
        self._client.auth = self._sign_request
        self.authenticated = True

    def start_test_run(self, project_key: str, body: StartTestRunModel) -> Optional[int]:
        url = self.service_url + "/api/reporting/v1/test-runs"

        try:
            response = self._client.post(
                url, params={"projectKey": project_key}, json=body.dict(exclude_none=True, by_alias=True)
            )
        except httpx.RequestError as e:
            logger.warning("Error while sending request to zebrunner.", exc_info=e)
            return None

        if response.status_code != 200:
            log_response(response, logging.ERROR)
            return None

        return response.json()["id"]

    def start_test(self, test_run_id: int, body: StartTestModel) -> Optional[int]:
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/tests"

        try:
            response = self._client.post(url, json=body.dict(exclude_none=True, by_alias=True))
        except httpx.RequestError as e:
            logger.warning("Error while sending request to zebrunner.", exc_info=e)
            return None

        if response.status_code != 200:
            log_response(response, logging.ERROR)
            return None

        return response.json()["id"]

    def finish_test(self, test_run_id: int, test_id: int, body: FinishTestModel) -> None:
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/tests/{test_id}"

        try:
            response = self._client.put(url, json=body.dict(exclude_none=True, by_alias=True))
        except httpx.RequestError as e:
            logger.warning("Error while sending request to zebrunner.", exc_info=e)
            return

        if response.status_code != 200:
            log_response(response, logging.ERROR)

    def finish_test_run(self, test_run_id: int) -> None:
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}"
        try:
            response = self._client.put(
                url,
                json={"endedAt": (datetime.utcnow().replace(tzinfo=timezone.utc) - timedelta(seconds=1)).isoformat()},
            )
        except httpx.RequestError as e:
            logger.warning("Error while sending request to zebrunner.", exc_info=e)
            return

        if response.status_code != 200:
            log_response(response, logging.ERROR)
            return

    def send_logs(self, test_run_id: int, logs: List[LogRecordModel]) -> None:
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/logs"

        body = [x.dict(exclude_none=True, by_alias=True) for x in logs]
        self._client.post(url, json=body)

    def send_screenshot(self, test_run_id: int, test_id: int, image_path: Union[str, Path]) -> None:
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/tests/{test_id}/screenshots"
        with open(image_path, "rb") as image:
            self._client.post(url, content=image.read(), headers={"Content-Type": "image/png"})

    def send_artifact(self, filename: Union[str, Path], test_run_id: int, test_id: Optional[int] = None) -> None:
        if test_id:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/tests/{test_id}/artifacts"
        else:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/artifacts"
        with open(filename, "rb") as file:
            self._client.post(url, files={"file": file})

    def send_artifact_references(
        self, references: List[ArtifactReferenceModel], test_run_id: int, test_id: Optional[int] = None
    ) -> None:
        if test_id:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/tests/{test_id}/artifact-references"
        else:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/artifact-references/"
        json_items = [item.dict(exclude_none=True, by_alias=True) for item in references]
        self._client.put(url, json={"items": json_items})

    def send_labels(self, labels: List[LabelModel], test_run_id: int, test_id: Optional[int] = None) -> None:
        if test_id:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/tests/{test_id}/labels"
        else:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/labels"
        labels_json = [label.dict(exclude_none=True, by_alias=True) for label in labels]
        self._client.put(url, json={"items": labels_json})

    def start_test_session(self, test_run_id: int, body: StartTestSessionModel) -> Optional[str]:
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/test-sessions"
        response = self._client.post(url, json=body.dict(exclude_none=True, by_alias=True))
        if not response.status_code == 200:
            log_response(response, logging.ERROR)
            return None

        return response.json().get("id")

    def finish_test_session(self, test_run_id: int, zebrunner_id: str, body: FinishTestSessionModel) -> None:
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/test-sessions/{zebrunner_id}"
        self._client.put(url, json=body.dict(exclude_none=True, by_alias=True))

    def close(self) -> None:
        self._client.close()



import logging
from typing import Any, Dict

from pytest_zebrunner.api.client import ZebrunnerAPI
from pytest_zebrunner.api.models import FinishTestSessionModel, StartTestSessionModel
from pytest_zebrunner.context import zebrunner_context

logger = logging.getLogger(__name__)


class SeleniumSession:
    def __init__(self, api: ZebrunnerAPI) -> None:
        self._active_sessions: Dict[str, Any] = {}
        self.api = api

    def start_session(self, session_id: str, capabilities: dict, desired_capabilities: dict) -> None:
        self._active_sessions[session_id] = {"related_tests": []}

        if zebrunner_context.test_run_is_active:
            zebrunner_session_id = self.api.start_test_session(
                zebrunner_context.test_run_id,
                StartTestSessionModel(
                    session_id=session_id, desired_capabilities=desired_capabilities, capabilities=capabilities
                ),
            )
            self._active_sessions[session_id]["zebrunner_session_id"] = zebrunner_session_id

    def finish_session(self, session_id: str) -> None:
        if zebrunner_context.test_run_is_active:
            self.api.finish_test_session(
                zebrunner_context.test_run_id,
                self._active_sessions[session_id]["zebrunner_session_id"],
                FinishTestSessionModel(test_ids=self._active_sessions[session_id]["related_tests"]),
            )
            del self._active_sessions[session_id]

    def finish_all_sessions(self) -> None:
        for session_id in list(self._active_sessions):
            self.finish_session(session_id)

    def add_test(self, test_id: int) -> None:
        for session_id in self._active_sessions:
            if self._active_sessions[session_id].get("related_tests") is not None:
                self._active_sessions[session_id]["related_tests"].append(test_id)
            else:
                self._active_sessions[session_id]["related_tests"] = [test_id]


def inject_driver(session_manager: SeleniumSession) -> None:
    try:
        from selenium.webdriver.remote.webdriver import WebDriver

        base_init = WebDriver.__init__
        base_close = WebDriver.close

        def init(session, *args, **kwargs) -> None:  # type: ignore
            base_init(session, *args, **kwargs)
            session_manager.start_session(session.session_id, session.capabilities, session.desired_capabilities)

        def close(session) -> None:  # type: ignore
            session_manager.finish_session(session.session_id)
            base_close(session)

        WebDriver.__init__ = init
        WebDriver.close = close

    except ImportError:
        logger.warning("Selenium library is not installed.")
